// (c) Copyright IdeaBlade, Inc. and Microsoft Corporation
// Derived from Microsoft Silverlight User Test Framework// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.

using System;
using System.Threading;

namespace IdeaBlade.TestFramework {
  /// <summary>
  /// A work item for tests to use which will call the delegate when the work 
  /// item is executed.
  /// </summary>
  public class CallbackWorkItem : SynchronizedWorkItem {
    /// <summary>
    /// Stored void delegate.
    /// </summary>
    private Action _action;

    /// <summary>
    /// Create a new Callback work item.
    /// </summary>
    /// <param name="callback">Action to execute on Invoke.</param>
    /// <param name="context">
    /// Optional <see cref="SynchronizationContext"/> in which callback should be executed.
    /// </param>
    public CallbackWorkItem(Action callback, SynchronizationContext context = null) {
      _action = callback;
      SyncContext = context;
    }


    /// <summary>
    /// Execute the callback.
    /// </summary>
    /// <returns>Whether the work item is complete (which it always is).</returns>
    public override bool Invoke() {
      Send(_action);
      WorkItemComplete();
      return base.Invoke();
    }
  }
}