using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using IdeaBlade.EntityModel;
using Microsoft.VisualStudio.TestTools.UnitTesting;
// ReSharper disable RedundantArgumentName

namespace IdeaBlade.TestFramework {
  // Desktop .NET specific portion of the DevForce Asynchronous test base class
  [TestClass]
  public partial class DevForceTest : AsyncTestMethodRunner {

    /// <summary>
    /// Ensure an EntityManager is pinned to the thread 
    /// of the test's SynchronizationContext.
    /// </summary>
    /// <remarks>
    /// Needed in Desktop; not needed in SL tests where Manager would be created in the SyncContext.
    /// </remarks>
    protected void PinManagerToSyncContextThread(EntityManager manager) {
      manager.AuthorizedThreadId = SyncContextThreadId;
    }

    partial void SetTestClassBase() {      
      // "WaitFor" extension methods refer to this instance on the SyncContext thread.
      EnqueueCallback(() => TestClassBase = this);
    }

    protected virtual void Log(string message) {
      Debug.WriteLine("*** [{0}] {1}", ++_logCount, message);
      _logMessages.Add(message);
    }

    private int _logCount;
    private readonly ObservableCollection<String> _logMessages = new ObservableCollection<string>();
  }
}