using System;
using System.Collections;
using System.IO;
using IdeaBlade.Core.Composition;
using IdeaBlade.EntityModel;
using Microsoft.VisualStudio.TestTools.UnitTesting;

// ReSharper disable InconsistentNaming
// ReSharper disable RedundantArgumentName

namespace IdeaBlade.TestFramework {
  public static class EntityModelTestExtensions {

    /// <summary>
    /// Reset the default FakeBackingStore with the <see cref="EntityCacheState"/>.
    /// </summary>
    public static void ResetDefaultFakeBackingStore(EntityCacheState ecs) {
#if SILVERLIGHT
      throw new InvalidOperationException("No LocalFakeBackingStore in Silverlight.");
#else
      EntityServerFakeBackingStore.Local.GetInstance(CompositionContext.Fake.Name).Restore(ecs);
#endif
    }
     
    /// <summary>
    /// Prevent Manager from talking to the server.
    /// </summary>
    /// <remarks>
    /// Overdetermined! Going all out to keep manager from talking to a server.
    /// </remarks>
    public static void ForceManagerLocal(this EntityManager manager) {
      manager.Disconnect();
      manager.BlockFetching();
      manager.BlockSaving();
    }

    // Throw if EntityManager attempts to fetch entities from server
    public static void BlockFetching(this EntityManager manager) {
      manager.Fetching -= _blockFetching;
      manager.Fetching += _blockFetching;
    }

    // Remove the block on fetching placed by <see cref="BlockFetching"/>.
    public static void UnblockFetching(this EntityManager manager) {
      manager.Fetching -= _blockFetching;
    }

    private static void _blockFetching(object sender, EntityFetchingEventArgs args) {
      throw new AssertFailedException(
        args.Query.EntityManager + " should not try to fetch data from the server.");
    }

    // Throw if EntityManager attempts to save 
    public static void BlockSaving(this EntityManager manager) {
      manager.Saving -= _blockSaving;
      manager.Saving += _blockSaving;
    }

    // Remove the block on saving placed by <see cref="BlockSaving"/>.
    public static void UnblockSaving(this EntityManager manager) {
      manager.Saving -= _blockSaving;
    }

    private static void _blockSaving(object sender, EntitySavingEventArgs args) {
      throw new AssertFailedException(
        "EntityManager should not try to save to the database.");
    }  
  }
}