﻿using System;
using System.ComponentModel.DataAnnotations;
using IdeaBlade.Extensions;
using IdeaBlade.Validation;

// ReSharper disable RedundantNameQualifier
#pragma warning disable 0649 // is never assigned to, and will always have its default value null

namespace Model
{
    // The MetadataTypeAttribute identifies CustomerMetadata as the class
    // that carries additional metadata for the Customer class.
    [MetadataType(typeof(Customer.CustomerMetadata))]
    public partial class Customer
    {
        public Customer()
        {
            CustomerID = SystemGuid.NewGuid();
        }

        // This class allows you to attach custom attributes to properties
        // of the Customer class.
        internal sealed class CustomerMetadata
        {
            // Metadata classes are not meant to be instantiated.
            private CustomerMetadata() { }

            // Changed Display name will show up in validation messages
            // Was [Display(Name = "CustomerId", AutoGenerateField = true)]
            [Display(Name = "Id", AutoGenerateField = true)]
            public Guid CustomerID { get; set; }

            [Unique(DisplayName = "Company Name")]
            public string CompanyName { get; set; }

            // Country is nullable in database but we want to require it
            [RequiredValueVerifier]
            public static string Country;

            // Was: [IbVal.StringLengthVerifier(MaxValue=15, IsRequired=false, ErrorMessageResourceName="Customer_City")]
            [System.ComponentModel.DataAnnotations.Required]
            [System.ComponentModel.DataAnnotations.StringLength(15)]
            public static string City;

        }

        // Custom properties and methods

        [Display(Name = "Original", AutoGenerateField = true, Order = 3)]
        public bool IsOriginal
        {
            get { return CustomerID_OLD != String.Empty; }
        }

        public bool CanDelete { get { return !IsOriginal; } }

        // Enforce integrity of order before adding it
        public void AddOrder(Order order)
        {
            // To Do: something real
            order.Customer = this;
        }

        // Enforce rules for removing an order
        public void RemoveOrder(Order order)
        {
            throw new InvalidOperationException("Can't remove an order");
        }

        // Easier debugging
        public override string ToString()
        {
            return String.Format("{0}({1})", CompanyName, CustomerID);
        }


    }
}
