﻿using System;
using IdeaBlade.Validation;
// ReSharper disable RedundantThisQualifier
namespace Model
{
    /// <summary>
    /// Ensure that the string property value is just the right length
    /// a custom verifier with a dynamic message.
    /// </summary>
    public class GoldilocksVerifier : PropertyValueVerifier
    {

        public GoldilocksVerifier(
            Type entityType, String propertyName, int justRight = 20,
            String displayName = null, bool? treatEmptyStringAsNull = null)
            : base(new PropertyValueVerifierArgs(
                entityType, propertyName, false,
                displayName, treatEmptyStringAsNull))
        {
            JustRight = justRight;
        }

        public int JustRight { get; set; }

        protected override VerifierResult VerifyValue(
            object itemToVerify, object valueToVerify,
            TriggerContext triggerContext, VerifierContext verifierContext)
        {
            var actualLength = ((string) valueToVerify).Length;

            // TODO: Localize the message

            if (actualLength == JustRight)
            {
                return new VerifierResult(true,
                    String.Format("The length of the {0}, {1}, is just right", 
                    this.DisplayName, actualLength));
            }

            var now = DateTime.Now;
            if (actualLength < JustRight)
            {
                return new VerifierResult(VerifierResultCode.Error,
                    String.Format("The length of the {0}, {1}, is too small at the moment [{2:t}]",
                    this.DisplayName, actualLength, now));
            }
            return new VerifierResult(VerifierResultCode.Error,
                String.Format("The length of the {0}, {1}, is too big at the moment [{2:t}]",
                this.DisplayName, actualLength, now));
        }
    }

}
