﻿using System;
using System.Linq;
using System.Resources;
using IdeaBlade.Extensions;
using IdeaBlade.Validation;
using SoftwareApproach.TestingExtensions;
using Model;
using Microsoft.VisualStudio.TestTools.UnitTesting;

// ReSharper disable InconsistentNaming
// ReSharper disable LocalizableElement
// ReSharper disable PossibleNullReferenceException

namespace ValidationLab
{
    [TestClass]
    public class When_using_DevForceResourceManagerEmulator
    {
        private ResourceManager _originalResourceManager;
        private VerifierEngine _verifierEngine;

        [TestInitialize]
        public void TestInitialize()
        {
            _verifierEngine = new VerifierEngine();
            _originalResourceManager = _verifierEngine.ErrorsResourceManager;

            // Set the ErrorsResourceManager for localization of validation error messages
            _verifierEngine.ErrorsResourceManager = CustomResources.ResourceManager;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            // Restore original global ErrorsResourceManager
            _verifierEngine.ErrorsResourceManager = _originalResourceManager;
        }

        [TestMethod]
        public void should_find_StringLengthVerifier_message_template()
        {
            DevForceResourceManagerEmulator.Instance.GetString("VerifierMaxStringLength")
                .ShouldNotBeNull();
        }

        [TestMethod]
        public void should_not_find_nonsense_message_template()
        {
            DevForceResourceManagerEmulator.Instance.GetString("nonsense")
                .ShouldBeNull();
        }

        [TestMethod]
        public void should_find_test_message_template()
        {
            DevForceResourceManagerEmulator.Instance.GetString("Test")
                .ShouldEqual(CustomResources.Test);
        }

        [TestMethod]
        public void should_find_Customer_ContactName_message_template()
        {
            DevForceResourceManagerEmulator.Instance.GetString("Customer_ContactName")
                .ShouldEqual(CustomResources.Customer_ContactName);
        }

        [TestMethod]
        public void ContactName_StringLengthVerifier_uses_custom_validation_message()
        {
            var aCustomer = CreateNewCustomer();
            aCustomer.ContactName = "A very very very very very very long contact name";

            var valResult = aCustomer.EntityAspect.ValidationErrors.FirstOrDefault();
            valResult.ShouldNotBeNull("Expected validation error");
            var expectedMessage = String.Format(CustomResources.Customer_ContactName,
                "ContactName", 30);
            valResult.Message.ShouldEqual(expectedMessage, "Unexpected message");
        }

        [TestMethod]
        public void Required_validation_message_is_replaced()
        {
            var aCustomer = CreateNewCustomer();
            aCustomer.CompanyName = null;

            var valResult = aCustomer.EntityAspect.ValidationErrors.FirstOrDefault();
            valResult.ShouldNotBeNull("Expected validation error");
            var expectedMessage = String.Format(CustomResources.VerifierRequired, "CompanyName");
            valResult.Message.ShouldEqual(expectedMessage, "Unexpected message");
        }

        private Customer CreateNewCustomer()
        {
            var manager = new NorthwindIBEntities(shouldConnect: false); // don't need server
            var aCustomer = new Customer { CompanyName = "Acme" };
            manager.AddEntity(aCustomer);
            aCustomer.EntityAspect.ValidationErrors.Any().ShouldBeFalse(
                "New customer should not have validation errors when created.");
            return aCustomer;
        }
    }
}
