﻿using System;
using System.Linq;
using IdeaBlade.EntityModel;
using IdeaBlade.Validation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Model;

// ReSharper disable InconsistentNaming
// ReSharper disable RedundantDefaultFieldInitializer
// ReSharper disable RedundantArgumentNameext;


namespace ValidationLab
{
    [TestClass]
    public class When_using_GoldilocksVerifier
    {
        [TestInitialize]
        public void TestInitialize()
        {
            manager = new EntityManager(shouldConnect: false);
            AddGoldilocksVerifier();
            CreateTestCustomer();
        }

        [TestMethod]
        public void ContactName_can_be_just_right()
        {
            testCustomer.ContactName = justRightContactName;
            Assert.AreEqual(String.Empty, FirstErrorMessage, "Unexpected validation error message");
        }

        [TestMethod]
        public void ContactName_can_be_too_small()
        {
            testCustomer.ContactName = "Baby Bear";
            var msg = FirstErrorMessage;
            Assert.IsTrue(msg.Contains("is too small"), String.Format("Unexpected message: '{0}'", msg));
        }

        [TestMethod]
        public void ContactName_can_be_too_big()
        {
            testCustomer.ContactName = "Mr. Papa Bear";
            var msg = FirstErrorMessage;
            Assert.IsTrue(msg.Contains("is too big"), String.Format("Unexpected message: '{0}'", msg));
        }

        [TestMethod]
        public void goldiVerifier_emits_just_right_message()
        {
            // Arrange: set to a valid ContactName
            testCustomer.ContactName = justRightContactName;

            // Act: Perform instance verification
            VerifierResultCollection results = manager.VerifierEngine.Execute(testCustomer);

            // Asserts on the GoldilocksVerifier's results
            var gvr = results.Where(r => r.Verifier == goldiVerifier).FirstOrDefault();

            Assert.IsNotNull(gvr, "Expected a GoldilocksVerifier result");
            var msg = gvr.Message;
            Assert.IsTrue(gvr.IsOk,
                String.Format("Got validation error, '{0}', when expected none", msg));
            Assert.IsTrue(msg.Contains("is just right"), String.Format("Unexpected message: '{0}'", msg));
        }

        #region Helpers

        private void AddGoldilocksVerifier()
        {
            justRightContactName = "Goldilocks";
            goldiVerifier = new GoldilocksVerifier(typeof (Customer), "ContactName");
            goldiVerifier.JustRight = justRightContactName.Length;

            manager.VerifierEngine.AddVerifier(goldiVerifier);
        }

        private void CreateTestCustomer()
        {
            testCustomer = new Customer
                               {
                                   CompanyName = "Acme",
                                   Country = "Freedonia",
                               };
            manager.AddEntity(testCustomer);
        }

        private string FirstErrorMessage
        {
            get
            {
                var err = testCustomer.EntityAspect.ValidationErrors.FirstOrDefault();
                return null == err ? String.Empty : err.Message;
            }
        }

        private GoldilocksVerifier goldiVerifier;
        private string justRightContactName;
        protected EntityManager manager;
        protected Customer testCustomer;

        #endregion

    }
}
