﻿using System;
using System.ComponentModel;
using System.Linq;
using IdeaBlade.Core.ComponentModel;
using IdeaBlade.EntityModel;
using IdeaBlade.Validation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Model;

// ReSharper disable InconsistentNaming
// ReSharper disable RedundantDefaultFieldInitializer
// ReSharper disable RedundantArgumentName

namespace ValidationLab
{
    [TestClass]
    public class When_validating_OrderDetail_in_cache
    {
        [TestInitialize]
        public void TestInitialize()
        {
            // Offline - all validation rule testing done locally (in cache)
            manager = new EntityManager(shouldConnect: false);

            testDetail = new OrderDetail
            {
                // OrderDetail's two-part key, faked
                OrderID = 1,
                ProductID = 1,
            };

            // Adding to manager makes entity available to a VerifierEngine
            manager.AddEntity(testDetail); 
        }

        [TestMethod]
        public void cannot_make_quantity_negative()
        {
            testDetail.Quantity = -1;
            var emsg = FirstErrorMessage;
            Assert.IsNotNull(emsg, "Expected a validation error message");
            Assert.AreEqual("Quantity must be greater than or equal to 0", emsg);
        }

        #region Helpers

        private NorthwindIBEntities Manager { get; set; }

        private string FirstErrorMessage
        {
            get
            {
                var err = testDetail.EntityAspect.ValidationErrors.FirstOrDefault();
                return null == err ? String.Empty : err.Message;
            }
        }

        protected EntityManager manager;
        protected OrderDetail testDetail;

        #endregion

    }

}
